/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  helpfunction.cpp
///  \brief 
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   01.10.2010
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <sys/stat.h>
#include "helpfunction.hpp"

namespace help
{
bool doesFileExists(std::string strFilename)
{
    struct stat stFileInfo;

    // Attempt to get the file attributes
    int intStat = stat(strFilename.c_str(), &stFileInfo);
    if (intStat == 0)
    {
        // We were able to get the file attributes
        // so the file obviously exists.
        return true;
    }
    else
    {
        // We were not able to get the file attributes.
        // This may mean that we don't have permission to
        // access the folder which contains this file. If you
        // need to do that level of checking, lookup the
        // return values of stat which will give you
        // more details on why stat failed.
        return false;
    }

    return false; // should never happen
}

My_Long_Int factorial(unsigned int a)
{
    if (a >= 2)
    {
        return a * factorial(a - 1);
    }
    return 1;
}

My_Long_Int binomial_coefficient(unsigned int n, unsigned int i)
{
    My_Long_Int res(factorial(n));
    res /= factorial(i);
    res /= factorial(n-i);

    return res;
}

My_Long_Int GaussBinomialBinary(unsigned int n, unsigned int k)
{
    My_Long_Int div(1);
    My_Long_Int res(1);
	for(unsigned int i=1; i<=k; ++i)
	{
		res *= (((My_Long_Int) 1)<<(n-i+1))-1;
		div *= (((My_Long_Int) 1)<< i )-1;
	}
	return res/div;
}

My_Long_Int GaussBinomial(unsigned int n, unsigned int k, unsigned int q)
{
    My_Long_Int res(1), div(1);
    My_Long_Int pow(1), pow_div(q);

    for(unsigned int i=0; i<n-k+1; ++i)
    {
        pow *= q;
    }

	for(unsigned int i=0; i<k; ++i, pow*=q, pow_div*=q)
	{
		res *= pow-1;
		div *= pow_div -1;
	}
	return res/div;
}

My_Long_Int power(unsigned int q, unsigned int n)
{
    My_Long_Int res(1);
    for(unsigned int i=0; i<n; i++)
    {
        res *= q;
    }
    return res;
}

//
//template<>
//My_Long_Int moduleQty<2>(unsigned int* type, unsigned int n, unsigned int q)
//{
//	if(q != 2)
//	{
//		std::cerr<<"not yet implemented"<<std::endl;
//	}
//	unsigned int k = type[0] + type[1];
//	My_Long_Int nec_qty = ((My_Long_Int) 1)<<( type[0] * (n - k) );
//	nec_qty *= GaussBinomialBinary( n - type[0] , type[1]);
//	nec_qty *= GaussBinomialBinary( n, type[0]);
//	return nec_qty;
//}
//
//template<>
//My_Long_Int moduleQty<1>(unsigned int* type, unsigned int n, unsigned int q)
//{
//	return GaussBinomial(n, type[0], q);
//}
}
