/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  ringelement.cpp
///  \brief 
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   04.10.2010
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "ringelement.hpp"
#include "field.hpp"

template<>
Element<Z4>& Element<Z4>::operator*=(const Element<Z4>& b)
{
	e = (e*b.e)%4;
	return *this;
}

template<>
Element<F2_F2>& Element<F2_F2>::operator*=(const Element<F2_F2>& b)
{
    e = (e*b.e)%4;
    return *this;
}

template<>
Element<Field>& Element<Field>::applyFrobenius(unsigned int pow)
{
    if(pow==0)
    {
        return *this;
    }
    e = Field::getFrobenius( pow )[e];
    return *this;
}

template<>
unsigned int Element<Field>::minimizeByFrobenius(unsigned int& pow)
{
    if(pow ==0)
    {
        return 0;
    }
    const std::vector<INDEX>& frob_img = Field::getFrobenius(pow);
    Element<Field> min(*this);
    Element<Field> cpy( frob_img[e] ); // once applied
    unsigned int min_exp = 0;
    unsigned int applied_exp = 1;

    while( cpy != *this)
    {
        if(cpy < min)
        {
            min=cpy;
            min_exp = applied_exp;
        }
        cpy.e =frob_img[cpy.e];
        ++applied_exp;
    }
    unsigned int result = min_exp*pow;
    pow *= applied_exp; // new stabilizer
    if(pow == Field::get_r())
    {
        pow = 0;
    }
    *this = min;
    return result; // the group element we did apply
}

template<>
const std::vector<unsigned int>& Element<Field>::getBasisRepresentation(void) const
{
    return Field::getBasisRepresentation(*this);
}

template<>
std::ostream& Element<Field>::printBasisRep(std::ostream& os,  char basis_symbol ) const
{
    const std::vector<unsigned int>& rep(getBasisRepresentation());
    std::vector<unsigned int>::const_iterator repIter(rep.begin());
    std::vector<unsigned int>::const_iterator repIterEnd(rep.end());

    os<<*repIter;
    ++repIter;

    for(unsigned int pow = 1; repIter != repIterEnd; ++pow, ++repIter )
    {
        os<<"+"<<*repIter<<"*"<<basis_symbol<<"^"<<pow;
    }
    return os;
}
